@@ -16,23 +16,23 @@ import zipfile
 import traceback
 import dateutil.parser
 import time
-
 import rarfile
+from collections import namedtuple
+
 try:
     from PIL import Image
-    from PIL import ImageFile
 except ImportError:
     import Image
-    import ImageFile
 
-__version__ = '0.13.2'
+__version__ = '1.0.0'
 
 PROGRAM_NAME = 'picopt'
 
 # Extensions
 REMOVE_EXT = '.%s-remove' % PROGRAM_NAME
 NEW_EXT = '.%s-optimized.png' % PROGRAM_NAME
-ARCHIVE_TMP_DIR_TEMPLATE = PROGRAM_NAME+'_tmp_%s'
+ARCHIVE_TMP_DIR_PREFIX = PROGRAM_NAME+'_tmp_'
+ARCHIVE_TMP_DIR_TEMPLATE = ARCHIVE_TMP_DIR_PREFIX+'%s'
 NEW_ARCHIVE_SUFFIX = '%s-optimized.cbz' % PROGRAM_NAME
 # Program args
 JPEGTRAN_ARGS = ['jpegtran', '-optimize']
@@ -55,6 +55,8 @@ CBR_FORMAT = 'CBR'
 COMIC_FORMATS = set((CBZ_FORMAT, CBR_FORMAT))
 FORMAT_DELIMETER = ','
 DEFAULT_FORMATS = 'ALL'
+NONE_FORMAT = 'NONE'
+ERROR_FORMAT = 'ERROR'
 # Programs
 PROGRAMS = ('optipng', 'pngout', 'jpegrescan', 'jpegtran', 'gifsicle',
             'advpng')
@@ -73,6 +75,10 @@ ABBREVS = (
     (1, 'bytes')
 )
 
+ReportStats = namedtuple('ReportStats', ['final_filename', 'bytes_diff',
+                                         'report_list'])
+ExtArgs = namedtuple('ExtArgs', ['old_filename', 'new_filename', 'arguments'])
+
 
 def humanize_bytes(num_bytes, precision=1):
     """
@@ -127,7 +133,7 @@ def does_external_program_run(prog, arguments):
         subprocess.call([prog, '-h'], stdout=null, stderr=null)
         result = True
     except OSError:
-        if arguments.verbose:
+        if arguments.verbose > 1:
             print("couldn't run %s" % prog)
         result = False
 
@@ -168,33 +174,36 @@ def get_arguments():
                         dest="recurse", default=0,
                         help="Recurse down through directories ignoring the"
                              "image file arguments on the command line")
-    parser.add_argument("-q", "--quiet", action="store_false",
+    parser.add_argument("-Q", "--quiet", action="store_false",
                         dest="verbose", default=1,
                         help="Do not display output")
-    parser.add_argument("-o", "--disable_optipng", action="store_false",
+    parser.add_argument("-v", "--verbose", action="count",
+                        dest="verbose", default=1,
+                        help="Do not display output")
+    parser.add_argument("-O", "--disable_optipng", action="store_false",
                         dest="optipng", default=1,
                         help="Do not optimize with optipng")
     parser.add_argument("-a", "--enable_advpng", action="store_true",
                         dest="advpng", default=0,
                         help="Optimize with advpng (disabled by default)")
-    parser.add_argument("-p", "--disable_pngout", action="store_false",
+    parser.add_argument("-P", "--disable_pngout", action="store_false",
                         dest="pngout", default=1,
                         help="Do not optimize with pngout")
-    parser.add_argument("-j", "--disable_jpegrescan", action="store_false",
+    parser.add_argument("-J", "--disable_jpegrescan", action="store_false",
                         dest="jpegrescan", default=1,
                         help="Do not optimize with jpegrescan")
-    parser.add_argument("-e", "--disable_progressive", action="store_false",
+    parser.add_argument("-E", "--disable_progressive", action="store_false",
                         dest="jpegtran_prog", default=1,
                         help="Don't try to reduce size by making "
                         "progressive JPEGs with jpegtran")
-    parser.add_argument("-t", "--disable_jpegtran", action="store_false",
+    parser.add_argument("-T", "--disable_jpegtran", action="store_false",
                         dest="jpegtran", default=1,
                         help="Do not optimize with jpegtran")
     parser.add_argument("-b", "--bigger", action="store_true",
                         dest="bigger", default=0,
                         help="Save optimized files that are larger than "
                              "the originals")
-    parser.add_argument("-n", "--noop", action="store_true",
+    parser.add_argument("-N", "--noop", action="store_true",
                         dest="test", default=0,
                         help="Do not replace files with optimized versions")
     parser.add_argument("-l", "--list", action="store_true",
@@ -203,10 +212,10 @@ def get_arguments():
     parser.add_argument("-c", "--comics", action="store_true",
                         dest="comics", default=0,
                         help="Also optimize comic book archives (cbz & cbr)")
-    parser.add_argument("-g", "--disable_gifsicle", action="store_false",
+    parser.add_argument("-G", "--disable_gifsicle", action="store_false",
                         dest="gifsicle", default=1,
                         help="disable optimizing animated GIFs")
-    parser.add_argument("-C", "--disable_convert_type", action="store_const",
+    parser.add_argument("-Y", "--disable_convert_type", action="store_const",
                         dest="to_png_formats",
                         const=PNG_CONVERTABLE_FORMATS, default=PNG_FORMATS,
                         help="Do not convert other lossless formats like "
@@ -219,25 +228,29 @@ def get_arguments():
     parser.add_argument("-D", "--optimize_after", action="store",
                         dest="optimize_after", default=None,
                         help="only optimize files after the specified "
-                             "timestamp. Supercedes -T")
-    parser.add_argument("-T", "--record_timestamp", action="store_true",
+                             "timestamp. Supercedes -t")
+    parser.add_argument("-t", "--record_timestamp", action="store_true",
                         dest="record_timestamp", default=0,
                         help="Store the time of the optimization of full "
                              "directories in directory local dotfiles.")
-    parser.add_argument("-v", "--version", action="version",
+    parser.add_argument("-V", "--version", action="version",
                         version=__version__,
                         help="display the version number")
-    parser.add_argument("-m", "--destroy_metadata", action="store_true",
+    parser.add_argument("-M", "--destroy_metadata", action="store_true",
                         dest="destroy_metadata", default=0,
                         help="*Destroy* metadata like EXIF and JFIF")
     parser.add_argument("paths", metavar="path", type=str, nargs="+",
                         help="File or directory paths to optimize")
 
-    arguments = parser.parse_args()
+    return parser.parse_args()
 
+
+def process_arguments(arguments):
+    """ Recomputer special cases for input arguments """
     program_reqs(arguments)
 
     arguments.paths = set(arguments.paths)
+    arguments.archive_name = None
 
     if arguments.formats == DEFAULT_FORMATS:
         extra_formats = JPEG_FORMATS | COMIC_FORMATS | GIF_FORMATS
@@ -277,8 +290,10 @@ def replace_ext(filename, new_ext):
     return new_filename
 
 
-def new_percent_saved(size_in, size_out):
-    """spits out how much space the optimazation saved"""
+def new_percent_saved(report_stats):
+    """spits out how much space the optimization saved"""
+    size_in = report_stats.bytes_diff['in']
+    size_out = report_stats.bytes_diff['out']
     percent_saved = (1 - (size_out / size_in)) * 100
 
     size_saved_kb = humanize_bytes(size_in - size_out)
@@ -291,53 +306,52 @@ def run_ext(args):
     subprocess.call(args, stdout=subprocess.PIPE)
 
 
-def pngout(filename, new_filename, arguments):
+def pngout(ext_args):
     """runs the EXTERNAL program pngout on the file"""
-    args = PNGOUT_ARGS + [filename, new_filename]
+    args = PNGOUT_ARGS + [ext_args.old_filename, ext_args.new_filename]
     run_ext(args)
 
 
-def optipng(filename, new_filename, arguments):
+def optipng(ext_args):
     """runs the EXTERNAL program optipng on the file"""
-    args = OPTIPNG_ARGS + [new_filename]
+    args = OPTIPNG_ARGS + [ext_args.new_filename]
     run_ext(args)
 
 
-def advpng(filename, new_filename, arguments):
+def advpng(ext_args):
     """runs the EXTERNAL program advpng on the file"""
-    args = ADVPNG_ARGS + [new_filename]
+    args = ADVPNG_ARGS + [ext_args.new_filename]
     run_ext(args)
 
 
-def gifsicle(filename, new_filename, arguments):
+def gifsicle(ext_args):
     """runs the EXTERNAL program gifsicle"""
-    args = GIFSICLE_ARGS + [new_filename]
-    if arguments.verbose:
-        print("You should really convert animated GIFS to HTML5 video")
+    args = GIFSICLE_ARGS + [ext_args.new_filename]
     run_ext(args)
 
 
-def jpegtran(filename, new_filename, arguments):
+def jpegtran(ext_args):
     """create argument list for jpegtran"""
     args = copy.copy(JPEGTRAN_ARGS)
-    if arguments.destroy_metadata:
+    if ext_args.arguments.destroy_metadata:
         args += ["-copy", "none"]
     else:
         args += ["-copy", "all"]
-    if arguments.jpegtran_prog:
+    if ext_args.arguments.jpegtran_prog:
         args += ["-progressive"]
     args += ['-outfile']
-    args += [new_filename, filename]
+    args += [ext_args.new_filename, ext_args.filename]
+    run_ext(args)
 
 
-def jpegrescan(filename, new_filename, arguments):
+def jpegrescan(ext_args):
     """runs the EXTERNAL program jpegrescan"""
     args = copy.copy(JPEGRESCAN_ARGS)
-    if arguments.jpegrescan_multithread:
+    if ext_args.arguments.jpegrescan_multithread:
         args += ['-t']
-    if arguments.destroy_metadata:
+    if ext_args.arguments.destroy_metadata:
         args += ['-s']
-    args += [filename, new_filename]
+    args += [ext_args.old_filename, ext_args.new_filename]
     run_ext(args)
 
 
@@ -362,8 +376,8 @@ def cleanup_after_optimize(filename, new_filename, arguments):
         bytes_diff['out'] = filesize_in  # overwritten on succes below
         if (filesize_out > 0) and ((filesize_out < filesize_in)
                                    or arguments.bigger):
-            old_image_format = get_image_format(filename)
-            new_image_format = get_image_format(new_filename)
+            old_image_format = get_image_format(filename, arguments)
+            new_image_format = get_image_format(new_filename, arguments)
             if old_image_format != new_image_format:
                 final_filename = replace_ext(filename,
                                              new_image_format.lower())
@@ -381,7 +395,7 @@ def cleanup_after_optimize(filename, new_filename, arguments):
     except OSError as ex:
         print(ex)
 
-    return bytes_diff, final_filename
+    return ReportStats._make([final_filename, bytes_diff, []])
 
 
 def optimize_image_external(filename, arguments, func):
@@ -389,130 +403,144 @@ def optimize_image_external(filename, arguments, func):
     new_filename = os.path.normpath(filename + NEW_EXT)
     shutil.copy2(filename, new_filename)
 
-    func(filename, new_filename, arguments)
+    ext_args = ExtArgs._make([filename, new_filename, arguments])
+    func(ext_args)
 
-    bytes_diff, final_filename = cleanup_after_optimize(filename,
-                                                        new_filename,
-                                                        arguments)
-    percent = new_percent_saved(bytes_diff['in'], bytes_diff['out'])
+    report_stats = cleanup_after_optimize(filename, new_filename, arguments)
+    percent = new_percent_saved(report_stats)
     if percent != 0:
         report = '%s: %s' % (func.__name__, percent)
     else:
         report = ''
-    return (bytes_diff, report, final_filename)
+    report_stats.report_list.append(report)
+
+    return report_stats
 
 
 def optimize_gif(filename, arguments):
     """run EXTERNAL programs to optimize animated gifs"""
     if arguments.gifsicle:
-        bytes_diff, rep, final_filename = optimize_image_external(
-            filename, arguments, gifsicle)
+        report_stats = optimize_image_external(filename, arguments,
+                                               gifsicle)
     else:
         rep = ['Skipping animated GIF: %s' % filename]
         bytes_diff = {'in': 0, 'out': 0}
+        report_stats = ReportStats._make([filename, bytes_diff, [rep]])
 
-    report_list = [rep]
-
-    return bytes_diff, report_list, final_filename
+    return report_stats
 
 
 def optimize_png(filename, arguments):
     """run EXTERNAL programs to optimize lossless formats to PNGs"""
-    bytes_diff = None
-    report_list = []
     final_filename = filename
 
     filesize_in = os.stat(filename).st_size
+    report_stats = None
 
     for ext_prog in ('optipng', 'advpng', 'pngout'):
         if not getattr(arguments, ext_prog):
             continue
-        bytes_diff, rep, final_filename = optimize_image_external(
+        report_stats = optimize_image_external(
             final_filename, arguments, globals()[ext_prog])
-        if rep:
-            report_list += [rep]
 
-    if bytes_diff is not None:
-        bytes_diff['in'] = filesize_in
+    if report_stats is not None:
+        report_stats.bytes_diff['in'] = filesize_in
     else:
-        report_list += ['Skipping PNG file: %s' % final_filename]
         bytes_diff = {'in': 0, 'out': 0}
+        rep = 'Skipping PNG file: %s' % final_filename
+        report_stats = ReportStats._make([final_filename, bytes_diff, rep])
 
-    return bytes_diff, report_list, final_filename
+    return report_stats
 
 
 def optimize_jpeg(filename, arguments):
     """run EXTERNAL programs to optimize jpeg formats"""
     final_filename = filename
     if arguments.jpegrescan:
-        bytes_diff, rep, final_filename = optimize_image_external(
+        report_stats = optimize_image_external(
             final_filename, arguments, jpegrescan)
     elif arguments.jpegtran_prog or arguments.jpegtran:
-        bytes_diff, rep, final_filename = optimize_image_external(
+        report_stats = optimize_image_external(
             final_filename, arguments, jpegtran)
     else:
         rep = ['Skipping JPEG file: %s' % filename]
         bytes_diff = {'in': 0, 'out': 0}
+        report_stats = ReportStats._make([filename, bytes_diff, [rep]])
 
-    report_list = [rep]
-
-    return bytes_diff, report_list, final_filename
+    return report_stats
 
 
 def optimize_image(arg):
     """optimizes a given image from a filename"""
     try:
-        filename, image_format, arguments, total_bytes_in, total_bytes_out = arg
-
-        #print(filename, image_format, "starting...")
+        filename, image_format, arguments, total_bytes_in, total_bytes_out, \
+            nag_about_gifs = arg
 
         if is_format_selected(image_format, arguments.to_png_formats,
-                              arguments, arguments.optipng or arguments.pngout):
-            bytes_diff, report_list, final_filename = optimize_png(
-                filename, arguments)
+                              arguments, arguments.optipng or
+                              arguments.pngout):
+            report_stats = optimize_png(filename, arguments)
         elif is_format_selected(image_format, JPEG_FORMATS, arguments,
                                 arguments.jpegrescan or arguments.jpegtran):
-            bytes_diff, report_list, final_filename = optimize_jpeg(
-                filename, arguments)
+            report_stats = optimize_jpeg(filename, arguments)
         elif is_format_selected(image_format, GIF_FORMATS, arguments,
                                 arguments.gifsicle):
             # this captures still GIFs too if not caught above
-            bytes_diff, report_list, final_filename = optimize_gif(
-                filename, arguments)
-
+            report_stats = optimize_gif(filename, arguments)
+            nag_about_gifs.set(True)
         else:
-            if arguments.verbose:
+            if arguments.verbose > 1:
                 print(filename, image_format)  # image.mode)
                 print("\tFile format not selected.")
             return
 
-        optimize_accounting(final_filename, bytes_diff, report_list,
-                            total_bytes_in, total_bytes_out, arguments)
+        optimize_accounting(report_stats, total_bytes_in, total_bytes_out,
+                            arguments)
     except Exception as exc:
         print(exc)
         traceback.print_exc(exc)
         raise exc
 
 
-def optimize_accounting(filename, bytes_diff, report_list, total_bytes_in,
-                        total_bytes_out, arguments):
+def truncate_cwd(full_filename, arguments):
+    """ remove the cwd from the full filenam """
+    truncated_filename = full_filename.split(arguments.dir, 1)[1]
+    truncated_filename = truncated_filename.split(os.sep, 1)[1]
+    return truncated_filename
+
+
+def optimize_accounting(report_stats, total_bytes_in, total_bytes_out,
+                        arguments):
     """record the percent saved, print it and add it to the totals"""
-    report = filename + ': '
-    total = new_percent_saved(bytes_diff['in'], bytes_diff['out'])
-    if total:
-        report += total
-    else:
-        report += '0%'
-    if arguments.test:
-        report += ' could be saved.'
-    tools_report = ', '.join(report_list)
-    if tools_report:
-        report += '\n\t' + tools_report
+    if arguments.verbose:
+        report = ''
+        if arguments.archive_name is not None:
+            truncated_filename = report_stats.final_filename.split(
+                ARCHIVE_TMP_DIR_PREFIX, 1)[1]
+            truncated_filename = truncated_filename.split(os.sep, 1)[1]
+            report += '  %s: ' % arguments.archive_name
+        elif arguments.dir in report_stats.final_filename:
+            truncated_filename = truncate_cwd(report_stats.final_filename,
+                                              arguments)
+        else:
+            truncated_filename = report_stats.final_filename
 
-    total_bytes_in.set(total_bytes_in.get() + bytes_diff['in'])
-    total_bytes_out.set(total_bytes_out.get() + bytes_diff['out'])
+        report += '%s: ' % truncated_filename
+        total = new_percent_saved(report_stats)
+        if total:
+            report += total
+        else:
+            report += '0%'
+        if arguments.test:
+            report += ' could be saved.'
+        if arguments.verbose > 1:
+            tools_report = ', '.join(report_stats.report_list)
+            if tools_report:
+                report += '\n\t' + tools_report
+        print(report)
 
-    print(report)
+    total_bytes_in.set(total_bytes_in.get() + report_stats.bytes_diff['in'])
+    total_bytes_out.set(total_bytes_out.get() + report_stats.bytes_diff['out'])
 
 
 def is_image_sequenced(image):
@@ -526,11 +554,11 @@ def is_image_sequenced(image):
     return result
 
 
-def get_image_format(filename):
+def get_image_format(filename, arguments):
     """gets the image format"""
     image = None
     bad_image = 1
-    image_format = 'NONE'
+    image_format = NONE_FORMAT
     sequenced = False
     try:
         image = Image.open(filename)
@@ -542,32 +570,31 @@ def get_image_format(filename):
 
     if sequenced:
         image_format = SEQUENCED_TEMPLATE % image_format
-    elif image is None or bad_image or image_format == 'NONE':
-        image_format = 'ERROR'
+    elif image is None or bad_image or image_format == NONE_FORMAT:
+        image_format = ERROR_FORMAT
         filename_ext = os.path.splitext(filename)[-1].lower()
         if filename_ext in COMIC_EXTS:
             if zipfile.is_zipfile(filename):
                 image_format = CBZ_FORMAT
             elif rarfile.is_rarfile(filename):
                 image_format = CBR_FORMAT
-        #TODO levels of verbosity
-#        if image_format == 'ERROR' and arguments.verbose and \
-#                not arguments.list_only:
-#            print(filename, "doesn't look like an image or comic archive.")
+        if (arguments.verbose > 1) and image_format == ERROR_FORMAT and \
+                (not arguments.list_only):
+            print(filename, "doesn't look like an image or comic archive.")
     return image_format
 
 
 def detect_file(filename, arguments):
     """decides what to do with the file"""
-    image_format = get_image_format(filename)
+    image_format = get_image_format(filename, arguments)
 
     if image_format in arguments.formats:
         return image_format
 
-    if image_format in ('NONE', 'ERROR'):
+    if image_format in (NONE_FORMAT, ERROR_FORMAT):
         return
 
-    if arguments.verbose and not arguments.list_only:
+    if arguments.verbose > 1 and not arguments.list_only:
         print(filename, image_format, 'is not a enabled image or '
                                       'comic archive type.')
 
@@ -591,7 +618,8 @@ def comic_archive_compress(args):
         #archive into new filename
         new_filename = replace_ext(filename, NEW_ARCHIVE_SUFFIX)
 
-        print('Rezipping archive', end='')
+        if arguments.verbose:
+            print('Rezipping archive', end='')
         with zipfile.ZipFile(new_filename, 'w',
                              compression=zipfile.ZIP_DEFLATED) as new_zf:
             root_len = len(os.path.abspath(tmp_dir))
@@ -600,21 +628,22 @@ def comic_archive_compress(args):
                 for fname in files:
                     fullpath = os.path.join(root, fname)
                     archive_name = os.path.join(archive_root, fname)
-                    print('.', end='')
+                    if arguments.verbose:
+                        print('.', end='')
                     new_zf.write(fullpath, archive_name, zipfile.ZIP_DEFLATED)
 
         # Cleanup tmpdir
         if os.path.isdir(tmp_dir):
-            print('.', end='')
+            if arguments.verbose:
+                print('.', end='')
             shutil.rmtree(tmp_dir)
-        print('done.')
+        if arguments.verbose:
+            print('done.')
 
-        bytes_diff, final_filename = cleanup_after_optimize(filename,
-                                                            new_filename,
-                                                            arguments)
-
-        optimize_accounting(final_filename, bytes_diff, [''],
-                            total_bytes_in, total_bytes_out, arguments)
+        report_stats = cleanup_after_optimize(filename, new_filename,
+                                              arguments)
+        optimize_accounting(report_stats, total_bytes_in, total_bytes_out,
+                            arguments)
     except Exception as exc:
         print(exc)
         traceback.print_exc(exc)
@@ -631,6 +660,10 @@ def comic_archive_uncompress(filename, image_format, arguments):
         bytes_diff = {'in': 0, 'out': 0}
         return (bytes_diff, report_list)
 
+    if arguments.verbose:
+        truncated_filename = truncate_cwd(filename, arguments)
+        print("Extracting %s..." % truncated_filename, end='')
+
     # create the tmpdir
     tmp_dir = get_archive_tmp_dir(filename)
     if os.path.isdir(tmp_dir):
@@ -650,6 +683,9 @@ def comic_archive_uncompress(filename, image_format, arguments):
         bytes_diff = {'in': 0, 'out': 0}
         return (bytes_diff, report_list)
 
+    if arguments.verbose:
+        print('done')
+
     return os.path.basename(tmp_dir)
 
 
@@ -731,11 +767,9 @@ def optimize_comic_archive(filename_full, image_format, arguments, multiproc,
     tmp_dir_basename = comic_archive_uncompress(filename_full,
                                                 image_format, arguments)
     # recurse into comic archive even if flag not set
-    if arguments.recurse:
-        archive_arguments = arguments
-    else:
-        archive_arguments = copy.deepcopy(arguments)
-        archive_arguments.recurse = True
+    archive_arguments = copy.deepcopy(arguments)
+    archive_arguments.recurse = True
+    archive_arguments.archive_name = os.path.basename(filename_full)
 
     # optimize contents of comic archive
     dirname = os.path.dirname(filename_full)
@@ -774,7 +808,7 @@ def optimize_file(filename_full, arguments, multiproc, optimize_after):
     else:
         # regular image
         args = [filename_full, image_format, arguments,
-                multiproc['in'], multiproc['out']]
+                multiproc['in'], multiproc['out'], multiproc['nag_about_gifs']]
         return multiproc['pool'].apply_async(optimize_image, args=(args,))
 
 
@@ -806,7 +840,7 @@ def optimize_files(cwd, filter_list, arguments, multiproc, optimize_after):
     return result_set
 
 
-def report_totals(bytes_in, bytes_out, arguments):
+def report_totals(bytes_in, bytes_out, arguments, nag_about_gifs):
     """report the total number and percent of bytes saved"""
     if bytes_in:
         bytes_saved = bytes_in - bytes_out
@@ -828,68 +862,108 @@ def report_totals(bytes_in, bytes_out, arguments):
                 msg = "Lost"
         msg += " a total of %s or %.*f%s" % (humanize_bytes(bytes_saved),
                                              2, percent_bytes_saved, '%')
-        print(msg)
-        if arguments.test:
-            print("Test run did not change any files.")
+        if arguments.verbose:
+            print(msg)
+            if arguments.test:
+                print("Test run did not change any files.")
 
     else:
-        print("Didn't optimize any files.")
+        if arguments.verbose:
+            print("Didn't optimize any files.")
+
+    if nag_about_gifs and arguments.verbose:
+        print("You should really convert animated GIFS to HTML5 video")
 
 
-def main():
-    """main"""
-    #TODO make the relevant parts of this call as a library
+def optimize_files_after(path, arguments, file_list, multiproc):
+    """ compute the optimize after date for the a batch of files
+        and then optimize them.
+    """
+    optimize_after = get_optimize_after(path, True, None, arguments)
+    return optimize_files(path, file_list, arguments, multiproc,
+                          optimize_after)
 
-    # Init
-    ImageFile.MAXBLOCK = 3000000  # default is 64k
-
-    arguments = get_arguments()
 
+def optimize_all_files(multiproc, arguments):
+    """ Optimize the files from the arugments list in two batches.
+        One for absolute paths which are probably outside the current
+        working directory tree and one for relative files.
+    """
+    # Change dirs
     os.chdir(arguments.dir)
     cwd = os.getcwd()
 
+    # Init records
+    record_dirs = set()
+    cwd_files = set()
+    full_result_set = set()
+
+    for filename in arguments.paths:
+        # Record dirs to put timestamps in later
+        if arguments.recurse and os.path.isdir(filename):
+            record_dirs.add(filename)
+
+        # Optimize all filenames that are not immediate descendants of
+        #   the cwd and compute their optimize-after times individually.
+        #   Otherwise add the files to the list to do next
+        path_dn, path_fn = os.path.split(os.path.realpath(filename))
+        if path_dn != cwd:
+            result_set = optimize_files_after(path_dn, arguments,
+                                              [path_fn], multiproc)
+            full_result_set.union(result_set)
+        else:
+            cwd_files.add(path_fn)
+
+    # Optimize immediate descendants with optimize after computed from
+    # the current directory
+    if len(cwd_files):
+        result_set = optimize_files_after(cwd, arguments, cwd_files,
+                                          multiproc)
+        full_result_set.union(result_set)
+
+    # Wait for all files to finish compressing
+    for result in full_result_set:
+        result.wait()
+
+    # Write timestamps
+    pool = multiproc['pool']
+    for filename in record_dirs:
+        args = (filename, arguments)
+        pool.apply_async(record_timestamp, args=(args,))
+
+
+def run_main(raw_arguments):
+    """ The main optimization call """
+
+    arguments = process_arguments(raw_arguments)
+
+    # Setup Multiprocessing
     manager = multiprocessing.Manager()
     total_bytes_in = manager.Value(int, 0)
     total_bytes_out = manager.Value(int, 0)
+    nag_about_gifs = manager.Value(bool, False)
     pool = multiprocessing.Pool()
 
-    multiproc = {'pool': pool, 'in': total_bytes_in, 'out': total_bytes_out}
-    record_dirs = set()
-    cwd_files = set()
+    multiproc = {'pool': pool, 'in': total_bytes_in, 'out': total_bytes_out,
+                 'nag_about_gifs': nag_about_gifs}
 
-    # Optimize
-    for filename in arguments.paths:
-
-        if arguments.recurse and os.path.isdir(filename):
-            # dirs to put timestamps in later
-            record_dirs.add(filename)
-
-        if os.path.isabs(filename):
-            # optimize absolute paths on the command line
-            abs_dir = os.path.dirname(filename)
-            optimize_after = get_optimize_after(abs_dir, True, None,
-                                                arguments)
-            optimize_files(abs_dir, [filename], arguments,
-                           multiproc, optimize_after)
-
-        else:
-            cwd_files.add(filename)
-
-    if len(cwd_files):
-        # optimize cwd files
-        optimize_after = get_optimize_after(cwd, True, None, arguments)
-        optimize_files(cwd, cwd_files, arguments, multiproc, optimize_after)
+    # Optimize Files
+    optimize_all_files(multiproc, arguments)
 
+    # Shut down multiprocessing
     pool = multiproc['pool']
     pool.close()
     pool.join()
 
-    # Finish up
-    for filename in record_dirs:
-        record_timestamp(filename, arguments)
-
+    # Finish by reporting totals
     report_totals(multiproc['in'].get(), multiproc['out'].get(),
-                  arguments)
+                  arguments, multiproc['nag_about_gifs'].get())
+
+
+def main():
+    """main"""
+    raw_arguments = get_arguments()
+    run_main(raw_arguments)
 
 
 if __name__ == '__main__':
